; 3d simplex noise written by randall,
; ported for KolibriOS and MenuetOS64 by macgub.
; (www.macgub.hekko.pl)
; Now is mixed SSE2 and 32 bit instructions.
Menuet = 1
Kolibri = 0
OS = Menuet



use32

	       org    0x0

	       db     'MENUET01'	      ; 8 byte id
	       dd     0x01		      ; header version
	       dd     START		      ; start of code
	       dd     IMG_END			; size of image
	       dd     I_END ;0x100000                ; memory for app
	       dd     I_END ;0xbffff                 ; esp
	       dd     0x0 , 0x0 	      ; I_Param , I_Icon

START:				; start of execution

     call main
     call draw_window
     call draw_from_buffer

still:

;    mov  eax,23                 ; wait here for event
;    mov  ebx,timeout
;    int  0x40
 ;   mov eax,11                   ; check for event no wait
    mov eax,10			; wait for event
    int 0x40

    cmp  eax,1			; redraw request ?
    je	 red
    cmp  eax,2			; key in buffer ?
    je	 key
    cmp  eax,3			; button in buffer ?
    je	 button

    jmp  noclose

  red:				; redraw
    call draw_window
    call draw_from_buffer
    jmp  still

  key:				; key
    mov  eax,2			; just read it and ignore
    int  0x40
    shr  eax,8
    cmp  eax, 27
    jne  still
    mov  eax, -1
    int  0x40


  button:			; button
    mov  eax,17 		; get id
    int  0x40

    cmp  ah,1			; button id=1 ?
    jne  noclose

    mov  eax,-1 		; close this program
    int  0x40
  noclose:

    jmp  still


draw_from_buffer:

	    mov eax,7
	    mov ebx,screen3
	    mov ecx,SIZE*65536+SIZE
      if OS = Kolibri
	    mov edx,0*65536+0
      else
	    mov edx,10*65536+25
      end if
	    int 0x40
ret


;-------------------------------------------------------------------------------
; NAME:         DOT3
; IN:           xmm0        | ? z0 y0 x0 |
; IN:           xmm1        | ? z1 y1 x1 |
; OUT:          xmm0        | s s s s | s = x0*x1+y0*y1+z0*z1
;-------------------------------------------------------------------------------
macro		DOT3	    {
		mulps	    xmm0,xmm1
		movaps	    xmm1,xmm0
		movaps	    xmm2,xmm0
		shufps	    xmm0,xmm0,0x00
		shufps	    xmm1,xmm1,0x55
		shufps	    xmm2,xmm2,0xaa
		addps	    xmm0,xmm1
		addps	    xmm0,xmm2
}
;-------------------------------------------------------------------------------
; NAME:         DOT4
; IN:           xmm0        | w0 z0 y0 x0 |
; IN:           xmm1        | w1 z1 y1 x1 |
; OUT:          xmm0        | s s s s | s = x0*x1+y0*y1+z0*z1+w0*w1
;-------------------------------------------------------------------------------
macro		DOT4	    {
		mulps	    xmm0,xmm1
		movaps	    xmm1,xmm0
		movaps	    xmm2,xmm0
		movaps	    xmm3,xmm0
		shufps	    xmm0,xmm0,0x00
		shufps	    xmm1,xmm1,0x55
		shufps	    xmm2,xmm2,0xaa
		shufps	    xmm3,xmm3,0xff
		addps	    xmm0,xmm1
		addps	    xmm2,xmm3
		addps	    xmm0,xmm2
}
;-------------------------------------------------------------------------------
; NAME:         FLOOR
; IN:           xmm0        | w z y x |
; OUT:          xmm0        | floor(w) floor(z) floor(y) floor(x) |
;-------------------------------------------------------------------------------
macro		FLOOR	    {
		cvttps2dq   xmm1,xmm0
		psrld	    xmm0,31
		psubd	    xmm1,xmm0
		cvtdq2ps    xmm0,xmm1
}
;-------------------------------------------------------------------------------
; NAME:         STEP
; IN:           xmm0        | ew ez ey ex | edge vector
; IN:           xmm1        | w z y x | value vector
; OUT:          xmm0        | step(ew,w) step(ez,z) step(ey,y) step(ex,x) |
;-------------------------------------------------------------------------------
macro		STEP	    {
		cmpltps     xmm1,xmm0
		andnps	    xmm1,dqword [g_1_0]
		movaps	    xmm0,xmm1
}
;-------------------------------------------------------------------------------
; NAME:         MOD289
; IN:           xmm0        | w z y x |
; OUT:          xmm0        | mod289(w) mod289(z) mod289(y) mod289(x) |
;                           mod289(s) = s - floor(s * (1.0/289.0)) * 289.0
;-------------------------------------------------------------------------------
macro		MOD289	    {
		movaps	    xmm2,xmm0
		mulps	    xmm0,dqword [g_1_div_289]
		FLOOR
		mulps	    xmm0,dqword [g_289_0]
		subps	    xmm2,xmm0
		movaps	    xmm0,xmm2
}
;-------------------------------------------------------------------------------
; NAME:         PERMUTE
; IN:           xmm0        | w z y x |
; OUT:          xmm0        | perm(w) perm(z) perm(y) perm(x) |
;                           perm(s) = mod289(((s*34.0)+1.0)*s)
;-------------------------------------------------------------------------------
macro		PERMUTE     {
		movaps	    xmm1,xmm0
		mulps	    xmm0,dqword [g_34_0]
		addps	    xmm0,dqword [g_1_0]
		mulps	    xmm0,xmm1
		MOD289
}
;-------------------------------------------------------------------------------
; NAME:         ABS
; IN:           xmm0        | w z y x |
; OUT:          xmm0        | abs(w) abs(z) abs(y) abs(x) |
;-------------------------------------------------------------------------------
macro		ABS	    {
		xorps	    xmm1,xmm1		; xmm1 = | 0 0 0 0 |
		subps	    xmm1,xmm0		; xmm1 = neg(x)
		maxps	    xmm0,xmm1		; xmm0 = abs(x)
}
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; NAME:         snoise3
; DESC:         3D Simplex noise (https://github.com/ashima/webgl-noise)
; IN:           xmm0        | ? z y x |
; OUT:          xmm0        | s s s s | s is noise value [-1.0,1.0]
;-------------------------------------------------------------------------------
align 16
snoise3:
v		equ	    ebp-16
i		equ	    ebp-32
x0		equ	    ebp-48
x1		equ	    ebp-64
x2		equ	    ebp-80
x3		equ	    ebp-96
i1		equ	    ebp-112
i2		equ	    ebp-128
xxmm8		equ	    ebp-144
xxmm15		equ	    ebp-160
xxmm14		equ	    ebp-176
xxmm13		equ	    ebp-192
temp		equ	    ebp-208
xxmm12		equ	    ebp-224
xxmm10		equ	    ebp-240

		push	    ebp
		mov	    ebp,esp
		sub	    esp,256
		movaps	    [v],xmm0		      ; save input on the stack
		;
		; Compute corners (x0, x1, x2, x3)
		;
		; i = floor(v + dot(v, C.yyy))
		movaps	    xmm1,dqword [g_snoise_C]
		shufps	    xmm1,xmm1,0x55	      ; xmm1 = C.yyy
		DOT3				      ; xmm0 = dot(xmm0,xmm1)
		addps	    xmm0,[v]
		FLOOR
		movaps	    [i],xmm0
		; x0 = v - i + dot(i, C.xxx)
		movaps	    xmm3,[v]
		subps	    xmm3,xmm0
		movaps	    xmm1,dqword [g_snoise_C]
		shufps	    xmm1,xmm1,0x00	      ; xmm1 = C.xxx
		DOT3				      ; xmm0 = dot(xmm0,xmm1)
		addps	    xmm3,xmm0
		movaps	    [x0],xmm3
		; compute i1 and i2
		movaps	    xmm0,xmm3
		shufps	    xmm0,xmm0,11001001b       ; xmm0 = | w x z y |
		movaps	    xmm1,xmm3
		STEP
		movaps	    xmm7,xmm0		      ; xmm7 = g
		movaps	    xmm6,dqword [g_1_0]
		subps	    xmm6,xmm7		      ; xmm6 = 1.0 - g = l
		shufps	    xmm6,xmm6,11010010b       ; xmm6 = | w y x z |
		movaps	    xmm0,xmm7
		movaps	    xmm1,xmm7
		minps	    xmm0,xmm6		      ; xmm0 = min(g.xyz, l.zxy)
		maxps	    xmm1,xmm6		      ; xmm1 = max(g.xyz, l.zxy)
		movaps	    [i1],xmm0		      ; xmm0 = i1
		movaps	    [i2],xmm1		      ; xmm1 = i2
		; compute x1, x2 and x3
		movaps	    xmm7,[x0]		      ; xmm7 = x0
		movaps	    xmm6,xmm7		      ; xmm6 = x0
		movaps	    xmm5,xmm7		      ; xmm5 = x0
		movaps	    xmm4,dqword [g_snoise_C]
		movaps	    xmm3,xmm4
		movaps	    xmm2,dqword [g_snoise_D]
		shufps	    xmm4,xmm4,0x00	      ; xmm4 = C.xxx
		shufps	    xmm3,xmm3,0x55	      ; xmm3 = C.yyy
		shufps	    xmm2,xmm2,0x55	      ; xmm2 = D.yyy
		subps	    xmm5,xmm0		      ; xmm5 = x0 - i1
		subps	    xmm6,xmm1		      ; xmm6 = x0 - i2
		subps	    xmm7,xmm2		      ; xmm7 = x0 - D.yyy
		addps	    xmm5,xmm4		      ; xmm5 = x0 - i1 + C.xxx
		addps	    xmm6,xmm3		      ; xmm6 = x0 - i2 + C.yyy
		movaps	    [x1],xmm5
		movaps	    [x2],xmm6
		movaps	    [x3],xmm7
		;
		; Compute permutations (p)
		;
		movaps	    xmm0,[i]
		MOD289
		movaps	    xmm7,xmm0
		movaps	    xmm6,xmm0
		movaps	    xmm5,xmm0
		shufps	    xmm7,xmm7,10101010b       ; xmm7 = i.zzzz
		shufps	    xmm6,xmm6,01010101b       ; xmm6 = i.yyyy
		shufps	    xmm5,xmm5,00000000b       ; xmm5 = i.xxxx
		movaps	    xmm4,[i1]		      ; xmm4 = i1
		movaps	    xmm3,[i2]		      ; xmm3 = i2
		;
		movaps	    xmm0,xmm4		      ; xmm0 = i1
		movaps	    xmm1,xmm3		      ; xmm1 = i2
		shufps	    xmm0,xmm0,10101010b       ; xmm0 = | i1.z i1.z i1.z i1.z |
		shufps	    xmm1,xmm1,10101010b       ; xmm1 = | i2.z i2.z i2.z i2.z |
		andps	    xmm0,dqword [g_mask_0010] ; xmm0 = | 0 0 i1.z 0 |
		andps	    xmm1,dqword [g_mask_0100] ; xmm1 = | 0 i2.z 0 0 |
		orps	    xmm0,xmm1		      ; xmm0 = | 0 i2.z i1.z 0 |
		orps	    xmm0,dqword [g_1_0_w]     ; xmm0 = | 1 i2.z i1.z 0 |
		addps	    xmm0,xmm7		      ; xmm0 = i.zzzz + | 1 i2.z i1.z 0 |
		PERMUTE
		movaps	    [xxmm8],xmm0		 ; xmm8 = p
		movaps	    xmm0,xmm4		      ; xmm0 = i1
		movaps	    xmm1,xmm3		      ; xmm1 = i2
		shufps	    xmm0,xmm0,01010101b       ; xmm0 = | i1.y i1.y i1.y i1.y |
		shufps	    xmm1,xmm1,01010101b       ; xmm1 = | i2.y i2.y i2.y i2.y |
		andps	    xmm0,dqword [g_mask_0010] ; xmm0 = | 0 0 i1.y 0 |
		andps	    xmm1,dqword [g_mask_0100] ; xmm1 = | 0 i2.y 0 0 |
		orps	    xmm0,xmm1		      ; xmm0 = | 0 i2.y i1.y 0 |
		orps	    xmm0,dqword [g_1_0_w]     ; xmm0 = | 1 i2.y i1.y 0 |
		addps	    xmm0,xmm6		      ; xmm0 = i.yyyy + | 1 i2.y i1.y 0 |
		addps	    xmm0,[xxmm8]
		PERMUTE
		movaps	    [xxmm8],xmm0		 ; xmm8 = p
		movaps	    xmm0,xmm4		      ; xmm0 = i1
		movaps	    xmm1,xmm3		      ; xmm1 = i2
		shufps	    xmm0,xmm0,00000000b       ; xmm0 = | i1.x i1.x i1.x i1.x |
		shufps	    xmm1,xmm1,00000000b       ; xmm1 = | i2.x i2.x i2.x i2.x |
		andps	    xmm0,dqword [g_mask_0010] ; xmm0 = | 0 0 i1.x 0 |
		andps	    xmm1,dqword [g_mask_0100] ; xmm1 = | 0 i2.x 0 0 |
		orps	    xmm0,xmm1		      ; xmm0 = | 0 i2.x i1.x 0 |
		orps	    xmm0,dqword [g_1_0_w]     ; xmm0 = | 1 i2.x i1.x 0 |
		addps	    xmm0,xmm5		      ; xmm0 = i.xxxx + | 1 i2.x i1.x 0 |
		addps	    xmm0,[xxmm8]
		PERMUTE
		movaps	    [xxmm8],xmm0		 ; xmm8 = p
		;
		; Compute gradients
		;
		movaps	    xmm0,dqword [g_snoise_D]
		movaps	    xmm1,xmm0
		shufps	    xmm0,xmm0,11100111b       ; xmm0 = | D.w D.z D.y D.w |
		shufps	    xmm1,xmm1,11001000b       ; xmm1 = | D.w D.x D.z D.x |
		mulps	    xmm0,dqword [g_1_div_7]
		subps	    xmm0,xmm1
		movaps	    xmm7,xmm0		      ; xmm7 = ns
		; xmm8 = j = p - 49.0 * floor(p * ns.z * ns.z)
		shufps	    xmm0,xmm0,10101010b       ; xmm0 = ns.zzzz
		mulps	    xmm0,xmm0
		mulps	    xmm0,[xxmm8]
		FLOOR
		mulps	    xmm0,dqword [g_49_0]
    ;            movaps      [temp],xmm3
		movaps	    xmm3,[xxmm8]
		subps	    xmm3,xmm0  ; xmm8 = j
		movaps	    [xxmm8],xmm3
    ;            movaps      xmm3,[temp]
		; x_ = floor(j * ns.zzzz)
		movaps	    xmm0,xmm7
		shufps	    xmm0,xmm0,10101010b       ; xmm0 = ns.zzzz
		mulps	    xmm0,[xxmm8]
		FLOOR				      ; xmm0 = x_
		movaps	    xmm6,xmm0		      ; xmm6 = x_
		; y_ = floor(j - 7.0 * x_)
		mulps	    xmm0,dqword [g_7_0]
		movaps	    xmm1,[xxmm8]
		subps	    xmm1,xmm0
		movaps	    xmm0,xmm1
		FLOOR
		movaps	    xmm5,xmm0		      ; xmm5 = y_
		; x = x_ * ns.xxxx + ns.yyyy
		; y = y_ * ns.xxxx + ns.yyyy
		movaps	    xmm0,xmm7		      ; xmm0 = ns
		movaps	    xmm1,xmm7		      ; xmm1 = ns
		shufps	    xmm0,xmm0,00000000b       ; xmm0 = ns.xxxx
		shufps	    xmm1,xmm1,01010101b       ; xmm1 = ns.yyyy
		mulps	    xmm6,xmm0		      ; xmm6 = x_ * ns.xxxx
		mulps	    xmm5,xmm0		      ; xmm5 = y_ * ns.xxxx
		addps	    xmm6,xmm1		      ; xmm6 = x = x_ * ns.xxxx + ns.yyyy
		addps	    xmm5,xmm1		      ; xmm5 = y = y_ * ns.xxxx + ns.yyyy
		; h = 1.0 - abs(x) - abs(y)
		movaps	    xmm4,dqword [g_1_0]       ; xmm4 = h = | 1 1 1 1 |
		movaps	    xmm0,xmm6		      ; xmm0 = x
		ABS				      ; xmm0 = abs(x)
		movaps	    xmm3,xmm0		      ; xmm3 = abs(x)
		movaps	    xmm0,xmm5		      ; xmm0 = y
		ABS				      ; xmm0 = abs(y)
		subps	    xmm4,xmm3		      ; xmm4 = h = 1.0 - abs(x)
		subps	    xmm4,xmm0		      ; xmm4 = h = 1.0 - abs(x) - abs(y)
		; b0 = vec4(x.xy, y.xy)
		movaps	    xmm0,xmm6		      ; xmm0 = x
		movaps	    xmm1,xmm5		      ; xmm1 = y
		unpcklps    xmm0,xmm1		      ; xmm0 = | y.y x.y y.x x.x |
		shufps	    xmm0,xmm0,11011000b       ; xmm0 = | y.y y.x x.y x.x |
		movaps	    xmm7,xmm0		      ; xmm7 = b0
		; b1 = vec4(x.zw, y.zw)
		movaps	    xmm0,xmm6		      ; xmm0 = x
		movaps	    xmm1,xmm5		      ; xmm1 = y
		unpckhps    xmm0,xmm1		      ; xmm0 = | y.w x.w y.z x.z |
		shufps	    xmm0,xmm0,11011000b       ; xmm0 = | y.w y.z x.w x.z |
		movaps	    xmm3,xmm0		      ; xmm3 = b1
		; s0 = floor(b0) * 2.0 + 1.0
		movaps	    xmm0,xmm7
		FLOOR
		addps	    xmm0,xmm0
		addps	    xmm0,dqword [g_1_0]
		movaps	    [xxmm15],xmm0		 ; xmm15 = s0
		; s1 = floor(b1) * 2.0 + 1.0
		movaps	    xmm0,xmm3
		FLOOR
		addps	    xmm0,xmm0
		addps	    xmm0,dqword [g_1_0]
		movaps	    [xxmm14],xmm0		 ; xmm14 = s1
		; sh = -step(h, vec4(0.0))
		movaps	    xmm0,xmm4		      ; xmm0 = h
		xorps	    xmm1,xmm1		      ; xmm1 = | 0 0 0 0 |
		STEP
		xorps	    xmm1,xmm1
		subps	    xmm1,xmm0
		movaps	    [xxmm13],xmm1		 ; xmm13 = sh
		; a0 = b0.xzyw + s0.xzyw * sh.xxyy
		shufps	    xmm7,xmm7,11011000b       ; xmm7 = b0 = | w y z x |
		movaps	    xmm0,[xxmm15]
		shufps	    xmm0,xmm0,11011000b     ; xmm15 = s0 = | w y z x |
		movaps	    [xxmm15],xmm0
		movaps	    xmm0,[xxmm13]
		shufps	    xmm0,xmm0,01010000b       ; xmm0 = | y y x x |
		mulps	    xmm0,[xxmm15]
		addps	    xmm0,xmm7
		movaps	    xmm7,xmm0		      ; xmm7 = a0
		; a1 = b1.xzyw + s1.xzyw * sh.zzww
		shufps	    xmm3,xmm3,11011000b       ; xmm3 = b1 = | w y z x |
		movaps	    xmm6,[xxmm14]
		shufps	    xmm6,xmm6,11011000b     ; xmm14 = s1 = | w y z x |
		movaps	    xmm5,[xxmm13]
		shufps	    xmm5,xmm5,11111010b     ; xmm13 = sh = | w w z z |
		mulps	    xmm6,xmm5
		addps	    xmm6,xmm3
	      ;  moaps       [xxmm14],xmm6
	      ;  movaps      xmm6,xmm14                ; xmm6 = a1
		; p0 = vec3(a0.xy, h.x)
		movaps	    xmm0,xmm7 ; xmm0 = a0
		shufps	    xmm0,xmm4,00000100b       ; | h.x h.x a0.y a0.x |
		movaps	    xmm5,xmm0		      ; xmm5 = p0
		; p1 = vec3(a0.zw, h.y)
		shufps	    xmm7,xmm4,01011110b       ; xmm7 = p1 = | h.y h.y a0.w a0.z |
		; p2 = vec3(a1.xy, h.z)
		movaps	    xmm0,xmm6 ; xmm0 = a1
		shufps	    xmm0,xmm4,10100100b       ; | h.z h.z a1.y a1.x |
		movaps	    xmm3,xmm0		      ; xmm3 = p2
		; p3 = vec3(a1.zw, h.w)
		shufps	    xmm6,xmm4,11111110b       ; xmm6 = p3 = | h.w h.w a1.w a1.z |
		;
		movaps	    xmm4,xmm3		      ; xmm4 = p2
		;
		; Normalize gradients
		;
		; xmm5 = p0, xmm7 = p1, xmm4 = p2, xmm6 = p3
		;
		; xmm15 = dot(p0, p0)
		movaps	    xmm0,xmm5		      ; xmm0 = p0
		movaps	    xmm1,xmm5		      ; xmm1 = p0
		DOT3
		movaps	    [xxmm15],xmm0		 ; xmm15 = dot(p0, p0)
		; xmm14 = dot(p1, p1)
		movaps	    xmm0,xmm7		      ; xmm0 = p1
		movaps	    xmm1,xmm7		      ; xmm1 = p1
		DOT3
		movaps	    [xxmm14],xmm0		 ; xmm14 = dot(p1, p1)
		; xmm13 = dot(p2, p2)
		movaps	    xmm0,xmm4		      ; xmm0 = p2
		movaps	    xmm1,xmm4		      ; xmm1 = p2
		DOT3
		movaps	    [xxmm13],xmm0		 ; xmm13 = dot(p2, p2)
		; xmm12 = dot(p3, p3)
		movaps	    xmm0,xmm6		      ; xmm0 = p3
		movaps	    xmm1,xmm6		      ; xmm1 = p3
		DOT3
		movaps	    [xxmm12],xmm0		 ; xmm12 = dot(p3, p3)
		;
		movaps	    xmm0,dqword [g_taylor_scale]
		movaps	    xmm1,dqword [g_taylor_bias]
	;        movaps      [temp],xmm3
		movaps	    xmm3,[xxmm15]
		mulps	    xmm3,xmm0
		movaps	    [xxmm15],xmm3
		movaps	    xmm3 ,[xxmm14]
		mulps	    xmm3,xmm0
		movaps	    [xxmm14],xmm3
		movaps	    xmm3,[xxmm13]
		mulps	    xmm3,xmm0
		movaps	    [xxmm13],xmm3
		movaps	    xmm3,[xxmm12]
		mulps	    xmm3,xmm0
		movaps	    [xxmm12],xmm3
		movaps	    xmm3,[xxmm15]
		addps	    xmm3,xmm1
		movaps	    [xxmm15],xmm3
		movaps	    xmm3,[xxmm14]
		addps	    xmm3,xmm1
		movaps	    [xxmm14],xmm3
		movaps	    xmm3,[xxmm13]
		addps	    xmm3,xmm1
		movaps	    [xxmm13],xmm3
		movaps	    xmm3,[xxmm12]
		addps	    xmm3,xmm1
		movaps	    [xxmm12],xmm3
		; normalize
		mulps	    xmm5,[xxmm15]		 ; xmm5 = p0
		mulps	    xmm7,[xxmm14]		 ; xmm7 = p1
		mulps	    xmm4,[xxmm13]		 ; xmm4 = p2
		mulps	    xmm6,[xxmm12]		 ; xmm6 = p3
       ;         movaps      xmm3,[temp]
		;
		; Mix final noise value
		;
		; xmm15 = dot(x0, x0)
		movaps	    xmm0,[x0]		      ; xmm0 = x0
		movaps	    xmm1,xmm0		      ; xmm1 = x0
		DOT3
		movaps	    [xxmm15],xmm0		 ; xmm15 = dot(x0, x0)
		; xmm14 = dot(x1, x1)
		movaps	    xmm0,[x1]		      ; xmm0 = x1
		movaps	    xmm1,xmm0		      ; xmm1 = x1
		DOT3
		movaps	    [xxmm14],xmm0		 ; xmm14 = dot(x1, x1)
		; xmm13 = dot(x2, x2)
		movaps	    xmm0,[x2]		      ; xmm0 = x2
		movaps	    xmm1,xmm0		      ; xmm1 = x2
		DOT3
		movaps	    [xxmm13],xmm0		 ; xmm13 = dot(x2, x2)
		; xmm12 = dot(x3, x3)
		movaps	    xmm0,[x3]		      ; xmm0 = x3
		movaps	    xmm1,xmm0		      ; xmm1 = x3
		DOT3
		movaps	    [xxmm12],xmm0		 ; xmm12 = dot(x3, x3)
		;
		movaps	    xmm3,[xxmm15]
		andps	    xmm3,dqword [g_mask_0001]
		movaps	    [xxmm15],xmm3
		movaps	    xmm3,[xxmm14]
		andps	    xmm3,dqword [g_mask_0010]
		movaps	    [xxmm14],xmm3
		movaps	    xmm3,[xxmm13]
		andps	    xmm3,dqword [g_mask_0100]
		movaps	    [xxmm13],xmm3
		movaps	    xmm3,[xxmm12]
		andps	    xmm3,dqword [g_mask_1000]
		movaps	    [xxmm12],xmm3
		movaps	    xmm3,[xxmm15]
		orps	    xmm3,[xxmm14]
		movaps	    [xxmm15],xmm3
		movaps	    xmm3,[xxmm13]
		orps	    xmm3,[xxmm12]
		movaps	    [xxmm13],xmm3
		movaps	    xmm3,[xxmm15]
		orps	    xmm3,[xxmm13]
		movaps	    [xxmm15],xmm3
		movaps	    xmm0,dqword [g_0_6]
		subps	    xmm0,[xxmm15]
		maxps	    xmm0,dqword [g_0_0]       ; xmm0 = m
		mulps	    xmm0,xmm0
		mulps	    xmm0,xmm0
		movaps	    [xxmm10],xmm0		 ; xmm10 = m^4
		;
		; xmm15 = dot(x0, p0)
		movaps	    xmm0,[x0]		      ; xmm0 = x0
		movaps	    xmm1,xmm5		      ; xmm1 = p0
		DOT3
		movaps	    [xxmm15],xmm0		 ; xmm15 = dot(x0, p0)
		; xmm14 = dot(x1, p1)
		movaps	    xmm0,[x1]		      ; xmm0 = x1
		movaps	    xmm1,xmm7		      ; xmm1 = p1
		DOT3
		movaps	    [xxmm14],xmm0		 ; xmm14 = dot(x1, p1)
		; xmm13 = dot(x2, p2)
		movaps	    xmm0,[x2]		      ; xmm0 = x2
		movaps	    xmm1,xmm4		      ; xmm1 = p2
		DOT3
		movaps	    [xxmm13],xmm0		 ; xmm13 = dot(x2, p2)
		; xmm12 = dot(x3, p3)
		movaps	    xmm0,[x3]		      ; xmm0 = x3
		movaps	    xmm1,xmm6		      ; xmm1 = p3
		DOT3
		movaps	    [xxmm12],xmm0		 ; xmm12 = dot(x3, p3)
		; put all above dots into xmm15
		movaps	    xmm3,[xxmm15]
		andps	    xmm3,dqword [g_mask_0001]
		movaps	    [xxmm15],xmm3
		movaps	    xmm3,[xxmm14]
		andps	    xmm3,dqword [g_mask_0010]
		movaps	    [xxmm14],xmm3
		movaps	    xmm3,[xxmm13]
		andps	    xmm3,dqword [g_mask_0100]
		movaps	    [xxmm13],xmm3
		movaps	    xmm3,[xxmm12]
		andps	    xmm3,dqword [g_mask_1000]
		movaps	    [xxmm12],xmm3
		movaps	    xmm3,[xxmm15]
		orps	    xmm3,[xxmm14]
		movaps	    [xxmm15],xmm3
		movaps	    xmm3,[xxmm13]
		orps	    xmm3,[xxmm12]
		movaps	    [xxmm13],xmm3
		movaps	    xmm3,[xxmm15]
		orps	    xmm3,[xxmm13]
		movaps	    [xxmm15],xmm3
		;
		movaps	    xmm0,[xxmm10]		 ; xmm0 = m^4
		movaps	    xmm1,[xxmm15]
		DOT4
		mulps	    xmm0,dqword [g_42_0]
		mov	    esp,ebp
		pop	    ebp
		ret
;-------------------------------------------------------------------------------
; NAME:         main
; DESC:         Program main function.
;-------------------------------------------------------------------------------
;align 16
main:
imgptr		equ	    ebp-8
rr13d		equ	    ebp-12
rr12d		equ	    ebp-16
		push	    ebp
		mov	    ebp,esp
		sub	    esp,128
		; alloc memory for the image
	       ; mov         eax,9                     ; sys_mmap
	       ; xor         edi,edi                   ; addr
	       ; mov         esi,SIZE*SIZE*4           ; length
	       ; mov         edx,0x1+0x2               ; PROT_READ | PROT_WRITE
	       ; mov         r10d,0x02+0x20            ; MAP_PRIVATE | MAP_ANONYMOUS
	       ; mov         r8,-1                     ; fd
	       ; xor         r9d,r9d                   ; offset
	       ; syscall
	       ; mov         [imgptr],rax
		mov	    ebx,screen3
		; begin loops
		;xor         r13d,r13d                 ; .LoopY index
	       mov	    dword[rr13d],0
.LoopY:
		;xor         r12d,r12d                 ; .LoopX index
	       mov	    dword[rr12d],0
.LoopX:
		; compute
		xorps	    xmm0,xmm0
		xorps	    xmm1,xmm1
		cvtsi2ss    xmm0,[rr12d]		 ; xmm0 = | 0 0 0 x |
		cvtsi2ss    xmm1,[rr13d]		 ; xmm1 = | 0 0 0 y |
		unpcklps    xmm0,xmm1		      ; xmm0 = | 0 0 y x |
		divps	    xmm0,dqword [g_size]
		addps	    xmm0,xmm0
		addps	    xmm0,xmm0
		call	    snoise3
		mulps	    xmm0,dqword [g_0_5]
		addps	    xmm0,dqword [g_0_5]
		; clamp to [0.0,1.0]
		minps	    xmm0,dqword [g_1_0]
		maxps	    xmm0,dqword [g_0_0]
		; convert from [0.0,1.0] to [0,255]
		mulps	    xmm0,dqword [g_255_0]
		cvttps2dq   xmm0,xmm0
		movd	    eax,xmm0
		mov	    [ebx+2],al		      ; red
		pshufd	    xmm1,xmm0,00000001b
		movd	    eax,xmm1
		mov	    [ebx+1],al		      ; green
		pshufd	    xmm1,xmm0,00000010b
		movd	    eax,xmm1
		mov	    [ebx+0],al		      ; blue
	      ;  mov         byte [ebx+3],255          ; alpha
		; advance pixel pointer
		add	    ebx,3
		; continue .LoopX
		inc	    dword[rr12d]
		cmp	    dword[rr12d],SIZE
		jne	    .LoopX
		; continue .LoopY
		inc	    dword[rr13d]
		cmp	    dword[rr13d],SIZE
		jne	    .LoopY
		; create TGA file
	    ;    mov         eax,85
	    ;    mov         rdi,g_tga_name
	    ;    mov         esi,110000000b
	    ;    syscall
	    ;    mov         rbx,rax
		; write header
	    ;    mov         eax,1
	    ;    mov         rdi,rbx
	    ;    mov         rsi,g_tga_head
	    ;    mov         edx,18
	    ;    syscall
		; write pixel data
	    ;    mov         eax,1
	    ;    mov         rdi,rbx
	    ;    mov         rsi,[imgptr]
	    ;    mov         edx,SIZE*SIZE*4
	    ;    syscall
		mov	    esp,ebp
		pop	    ebp
		ret
;-------------------------------------------------------------------------------
; NAME:         Debug
;-------------------------------------------------------------------------------
;align 16
;Debug:
;v               equ         rbp-16
;                push        rbp
;                mov         rbp,rsp
;                sub         rsp,128
;                mov         dword [v+0],1.2
;                mov         dword [v+4],2.4
;                mov         dword [v+8],3.5
;                mov         dword [v+12],0.0
;                movaps      xmm0,[v]
;                call        snoise3
;                mov         rsp,rbp
;                pop         rbp
;                ret
;                restore     v
;-------------------------------------------------------------------------------
; NAME:         _start
; DESC:         Program entry point.
;-------------------------------------------------------------------------------
;start:
;                call        main
;                ; terminate process
;                mov         eax,60
;                xor         edi,edi
;                syscall
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;   *********************************************
;   *******  WINDOW DEFINITIONS AND DRAW ********
;   *********************************************
draw_window:

    mov  eax,12 		   ; function 12:tell os about windowdraw
    mov  ebx,1			   ; 1, start of draw
    int  0x40
								   ; DRAW WINDOW
    mov  eax,0	; function 0 : define and draw window
  if OS = Kolibri
    mov  ebx,100*65536+SIZE+9	   ; [x start] *65536 + [x size]
    mov  ecx,100*65536+SIZE+25	   ; [y start] *65536 + [y size]
  else
    mov  ebx,100*65536+SIZE+18	    ; [x start] *65536 + [x size]
    mov  ecx,100*65536+SIZE+34	   ; [y start] *65536 + [y size]
  end if
    mov  edx,0x74000000 		   ; color of work area RRGGBB,8->color gl
    mov  edi,labelt
    int  0x40

    ; WINDOW LABEL
    mov     eax,4	    ; function 4 : write text to window
    mov     ebx,8*65536+8   ; [x start] *65536 + [y start]
    mov     ecx,0x20ddeeff  ; font 1 & color ( 0xF0RRGGBB )
    mov     edx,labelt	    ; pointer to text beginning
    mov     esi,labelen-labelt	   ; text length
    int     0x40

				   ; WINDOW LABEL
    mov  eax,12 		   ; function 12:tell os about windowdraw
    mov  ebx,2			   ; 2, end of draw
    int  0x40

    ret



;-------------------------------------------------------------------------------
;align 1
labelt:
 db  '3d simplex noise',0
labelen:

;-------------------------------------------------------------------------------
;align 1
;g_tga_name      db          'snoise.tga',0
;g_tga_head      db          0,0,2,9 dup 0
;                db          (SIZE and 0x00ff),(SIZE and 0xff00) shr 8
;                db          (SIZE and 0x00ff),(SIZE and 0xff00) shr 8,32,0
align 16
SIZE=600
g_size		dd	    4 dup 600.0

g_snoise_C	dd	    0.166666667,0.333333333,0.0,0.0
g_snoise_D	dd	    0.0,0.5,1.0,2.0

g_0_0		dd	    4 dup 0.0
g_0_5		dd	    4 dup 0.5
g_0_6		dd	    4 dup 0.6
g_1_0		dd	    4 dup 1.0
g_7_0		dd	    4 dup 7.0
g_34_0		dd	    4 dup 34.0
g_42_0		dd	    4 dup 42.0
g_49_0		dd	    4 dup 49.0
g_255_0 	dd	    4 dup 255.0
g_289_0 	dd	    4 dup 289.0
g_1_div_7	dd	    4 dup 0.142857142857
g_1_div_289	dd	    4 dup 0.003460208
g_mask_0001	dd	    0xffffffff,0x00000000,0x00000000,0x00000000
g_mask_0010	dd	    0x00000000,0xffffffff,0x00000000,0x00000000
g_mask_0100	dd	    0x00000000,0x00000000,0xffffffff,0x00000000
g_mask_1000	dd	    0x00000000,0x00000000,0x00000000,0xffffffff
g_1_0_w 	dd	    0.0,0.0,0.0,1.0
g_taylor_bias	dd	    4 dup 1.79284291400159
g_taylor_scale	dd	    4 dup -0.85373472095314
;-------------------------------------------------------------------------------
IMG_END:
align 16
;screen4 rb SIZE*SIZE*4
screen3 rb SIZE*SIZE*3
memStack  rb 1024
I_END: